/*
    This file is part of UDPserver.
    Copyright (c) 2024 Aleksanrds Grablevskis

    UDPserver is free software: you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation, either version 3 of the License, or
    (at your option) any later version.

    UDPserver is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with UDPserver. If not, see <https://www.gnu.org/licenses/>.
*/

#include <stdio.h>
#include <unistd.h>
#include <stdlib.h>
#include <string.h>
#include <netdb.h>
#include <sys/types.h>
#include <sys/socket.h>
#include <netinet/in.h>
#include <arpa/inet.h>
#include "config.h"

void error(char *msg){
  perror(msg);
  exit(1);
}

int main(int argc, char* argv[]){
  int sockfd;				/* socket */
  int portno;				/* port to listen on */
  int clientlen;			/* byte size of client's address */
  struct sockaddr_in serveraddr;	/* server's addr */
  struct sockaddr_in clientaddr;	/* client addr */
  char *buf;				/* message buf */
  int optval;				/* flag value for setsockopt */
  int n;				/* message byte size */
  int port;				/* client port */

  if(argc != 2){
    fprintf(stderr, "usage: %s <port>\n", argv[0]);
    exit(1);
  }
  printf("UDP server(%s)\n", APP_VERSION);
  portno = atoi(argv[1]);

  // socket: create the parent socket 
  sockfd = socket(AF_INET, SOCK_DGRAM, 0);
  if(sockfd < 0) error("ERROR opening socket");

  optval = 1;
  setsockopt(sockfd, SOL_SOCKET, SO_REUSEADDR, (const void *)&optval, sizeof(int));

  // build the server's Internet address
  bzero((char *)&serveraddr, sizeof(serveraddr));
  serveraddr.sin_family = AF_INET;
  serveraddr.sin_addr.s_addr = htonl(INADDR_ANY);
  serveraddr.sin_port = htons((unsigned short)portno);

  // bind: associate the parent socket with a port 
  if(bind(sockfd, (struct sockaddr *)&serveraddr, sizeof(serveraddr)) < 0) error("ERROR on binding");

  // main loop: wait for a datagram, then echo it
  clientlen = sizeof(clientaddr);
  buf = malloc(UDP_BUF_SIZE);
  while(1){
    n = recvfrom(sockfd, buf, UDP_BUF_SIZE, 0, (struct sockaddr *)&clientaddr, &clientlen);
    if(n < 0) error("ERROR in recvfrom");
    if(n >= UDP_BUF_SIZE - 1) error("ERROR in recvfrom");
    *(buf + n - 1) = 0;

    unsigned char *bytes = (unsigned char *) &clientaddr.sin_addr;

    port = htons(clientaddr.sin_port);

    printf("Received (%d bytes) from %d.%d.%d.%d:%i | %s\n", n, bytes[0], bytes[1], bytes[2], bytes[3], port, buf);

//    n = sendto(sockfd, buf, n, 0, (struct sockaddr *)&clientaddr, clientlen);
//    if(n < 0) error("ERROR in sendto");
  }
  free(buf);
}
